/*
 * Copyright (C) 2008 Emweb bvba, Kessel-Lo, Belgium.
 *
 * See the LICENSE file for terms of use.
 */

#include "Wt/WApplication"
#include "Wt/WVBoxLayout"
#include "Wt/WContainerWidget"
#include "Wt/WEnvironment"
#include "Wt/WString"
#include "Wt/WTree"
#include "Wt/WTreeTable"
#include "Wt/WText"
#include "Wt/WTreeTableNode"

namespace Wt {

WTreeTable::WTreeTable(WContainerWidget *parent)
  : WCompositeWidget(parent)
{
  setImplementation(impl_ = new WContainerWidget());

  WApplication *app = WApplication::instance();

  WVBoxLayout *layout = new WVBoxLayout();
  layout->setSpacing(0);
  layout->setContentsMargins(0, 0, 0, 0);

  layout->addWidget(headers_ = new WContainerWidget()); 
  headers_->setStyleClass("header");
  headers_->resize(WLength(100, WLength::Percentage), WLength());

  headerContainer_ = new WContainerWidget(headers_);
  headerContainer_->setFloatSide(Right);  

  headers_->addWidget(new WText(""));
  columnWidths_.push_back(WLength());

  WContainerWidget *content = new WContainerWidget();
  if (!app->environment().agentIE())
    content->setOverflow(WContainerWidget::OverflowAuto);
  else
    content->setAttributeValue
      ("style", "overflow-y: auto; overflow-x: hidden; zoom: 1");
  content->addWidget(tree_ = new WTree());

  layout->addWidget(content, 1);
  impl_->setLayout(layout);

  tree_->setMargin(3, Top);
  tree_->resize(WLength(100, WLength::Percentage), WLength());

  app->addAutoJavaScript
    ("var e=" + content->jsRef() + ";"
     "if (e) {"
     """var tw=" + jsRef() + ".offsetWidth;"
     """if (tw > 0) {"
     ""  "var h=" + headers_->jsRef() + ";"
     ""  "var c=" + content->jsRef() + ";"
     ""  "if (e.scrollHeight > e.offsetHeight) {"
     ""    "h.style.width=(tw - 16) + 'px';"
     + (app->environment().agentIE() ? 
	"c.style." + std::string(app->environment().agentIE6() ?
		      "marginRight" : "paddingRight") + "='16px';" : "") +
     ""  "} else {"
     ""    "h.style.width=tw + 'px';"
     + (app->environment().agentIE() ? 
	"c.style." + std::string(app->environment().agentIE6() ?
		      "marginRight" : "paddingRight") + "='0px';" : "") +
     ""  "}"
     """}"
     "}"
     );
}

WWidget *WTreeTable::headerWidget() const
{
  return headers_;
}

void WTreeTable::setTreeRoot(WTreeTableNode *root, const WString& h)
{
  tree_->setTreeRoot(root);
  header(0)->setText(h);
  root->setTable(this);
}

WTreeTableNode *WTreeTable::treeRoot()
{
  return dynamic_cast<WTreeTableNode *>(tree_->treeRoot());
}

void WTreeTable::setTree(WTree *root, const WString& h)
{
  if (tree_)
    delete tree_;

  header(0)->setText(h);
  impl_->addWidget(tree_ = new WTree());
  tree_->resize(WLength(100, WLength::Percentage), WLength());

  treeRoot()->setTable(this);
}

void WTreeTable::addColumn(const WString& header, const WLength& width)
{
  WText *t = new WText(header);
  t->resize(width, WLength());
  t->setInline(false);
  t->setFloatSide(Left);
  headerContainer_->addWidget(t);

  columnWidths_.push_back(width);
}

WText *WTreeTable::header(int column) const
{
  if (column == 0)
    return dynamic_cast<WText *>
      (dynamic_cast<WContainerWidget *>(impl_->children()[0])
       ->children()[1]);
  else
    return dynamic_cast<WText *>(headerContainer_->children()[column - 1]);
}

}
